@@ -20,6 +20,12 @@ package org.apache.uniffle.shuffle.manager;
 import java.util.LinkedHashSet;
 import java.util.Set;
 
+import org.apache.spark.shuffle.ShuffleHandleInfo;
+
+import org.apache.uniffle.common.ShuffleServerInfo;
+
+import static org.mockito.Mockito.mock;
+
 public class DummyRssShuffleManager implements RssShuffleManagerInterface {
   public Set<Integer> unregisteredShuffleIds = new LinkedHashSet<>();
 
@@ -47,4 +53,24 @@ public class DummyRssShuffleManager implements RssShuffleManagerInterface {
   public void unregisterAllMapOutput(int shuffleId) {
     unregisteredShuffleIds.add(shuffleId);
   }
+
+  @Override
+  public ShuffleHandleInfo getShuffleHandleInfoByShuffleId(int shuffleId) {
+    return null;
+  }
+
+  @Override
+  public void addFailuresShuffleServerInfos(String shuffleServerId) {}
+
+  @Override
+  public boolean reassignAllShuffleServersForWholeStage(
+      int stageId, int stageAttemptNumber, int shuffleId, int numMaps) {
+    return false;
+  }
+
+  @Override
+  public ShuffleServerInfo reassignFaultyShuffleServerForTasks(
+      int shuffleId, Set<Integer> partitionIds, String faultyShuffleServerId) {
+    return mock(ShuffleServerInfo.class);
+  }
 }
