@@ -49,7 +49,6 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.client.api.ShuffleReadClient;
 import org.apache.uniffle.client.factory.ShuffleClientFactory;
-import org.apache.uniffle.client.request.CreateShuffleReadClientRequest;
 import org.apache.uniffle.client.util.RssClientConfig;
 import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
@@ -96,7 +95,8 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
       Roaring64NavigableMap taskIdBitmap,
       ShuffleReadMetrics readMetrics,
       RssConf rssConf,
-      ShuffleDataDistributionType dataDistributionType) {
+      ShuffleDataDistributionType dataDistributionType,
+      Map<Integer, List<ShuffleServerInfo>> allPartitionToServers) {
     this.appId = rssShuffleHandle.getAppId();
     this.startPartition = startPartition;
     this.endPartition = endPartition;
@@ -114,7 +114,7 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
     this.taskIdBitmap = taskIdBitmap;
     this.hadoopConf = hadoopConf;
     this.readMetrics = readMetrics;
-    this.partitionToShuffleServers = rssShuffleHandle.getPartitionToServers();
+    this.partitionToShuffleServers = allPartitionToServers;
     this.rssConf = rssConf;
     this.dataDistributionType = dataDistributionType;
   }
@@ -246,23 +246,33 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
         boolean expectedTaskIdsBitmapFilterEnable =
             !(mapStartIndex == 0 && mapEndIndex == Integer.MAX_VALUE)
                 || shuffleServerInfoList.size() > 1;
-        CreateShuffleReadClientRequest request =
-            new CreateShuffleReadClientRequest(
-                appId,
-                shuffleId,
-                partition,
-                basePath,
-                1,
-                partitionNum,
-                partitionToExpectBlocks.get(partition),
-                taskIdBitmap,
-                shuffleServerInfoList,
-                hadoopConf,
-                dataDistributionType,
-                expectedTaskIdsBitmapFilterEnable,
-                rssConf);
+        int retryMax =
+            rssConf.getInteger(
+                RssClientConfig.RSS_CLIENT_RETRY_MAX,
+                RssClientConfig.RSS_CLIENT_RETRY_MAX_DEFAULT_VALUE);
+        long retryIntervalMax =
+            rssConf.getLong(
+                RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX,
+                RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX_DEFAULT_VALUE);
         ShuffleReadClient shuffleReadClient =
-            ShuffleClientFactory.getInstance().createShuffleReadClient(request);
+            ShuffleClientFactory.getInstance()
+                .createShuffleReadClient(
+                    ShuffleClientFactory.newReadBuilder()
+                        .appId(appId)
+                        .shuffleId(shuffleId)
+                        .partitionId(partition)
+                        .basePath(basePath)
+                        .partitionNumPerRange(1)
+                        .partitionNum(partitionNum)
+                        .blockIdBitmap(partitionToExpectBlocks.get(partition))
+                        .taskIdBitmap(taskIdBitmap)
+                        .shuffleServerInfoList(shuffleServerInfoList)
+                        .hadoopConf(hadoopConf)
+                        .shuffleDataDistributionType(dataDistributionType)
+                        .expectedTaskIdsBitmapFilterEnable(expectedTaskIdsBitmapFilterEnable)
+                        .retryMax(retryMax)
+                        .retryIntervalMax(retryIntervalMax)
+                        .rssConf(rssConf));
         RssShuffleDataIterator<K, C> iterator =
             new RssShuffleDataIterator<>(
                 shuffleDependency.serializer(), shuffleReadClient, readMetrics, rssConf);
