@@ -21,6 +21,14 @@ import java.nio.ByteBuffer;
 
 public class NoOpCodec extends Codec {
 
+  private static class LazyHolder {
+    static final NoOpCodec INSTANCE = new NoOpCodec();
+  }
+
+  public static NoOpCodec getInstance() {
+    return LazyHolder.INSTANCE;
+  }
+
   @Override
   public void decompress(ByteBuffer src, int uncompressedLen, ByteBuffer dest, int destOffset) {
     dest.put(src);
@@ -34,4 +42,16 @@ public class NoOpCodec extends Codec {
     System.arraycopy(src, 0, dst, 0, src.length);
     return dst;
   }
+
+  @Override
+  public int compress(ByteBuffer src, ByteBuffer dest) {
+    int destOff = dest.position();
+    dest.put(src.duplicate());
+    return dest.position() - destOff;
+  }
+
+  @Override
+  public int maxCompressedLength(int sourceLength) {
+    return sourceLength;
+  }
 }
