@@ -17,13 +17,24 @@
 
 package org.apache.uniffle.common.config;
 
+import org.apache.uniffle.common.ClientType;
 import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.compression.Codec;
 import org.apache.uniffle.common.netty.IOMode;
+import org.apache.uniffle.common.util.BlockIdLayout;
 
 import static org.apache.uniffle.common.compression.Codec.Type.LZ4;
 
 public class RssClientConf {
+  /**
+   * The prefix key for Hadoop conf. For Spark like that:
+   *
+   * <p>key: spark.rss.hadoop.fs.defaultFS val: hdfs://rbf-x1
+   *
+   * <p>The key will be extracted to the hadoop conf: "fs.defaultFS" and inject this into Hadoop
+   * storage configuration.
+   */
+  public static final String HADOOP_CONFIG_KEY_PREFIX = "rss.hadoop.";
 
   public static final ConfigOption<Codec.Type> COMPRESSION_TYPE =
       ConfigOptions.key("rss.client.io.compression.codec")
@@ -47,6 +58,36 @@ public class RssClientConf {
               "The type of partition shuffle data distribution, including normal and local_order. "
                   + "The default value is normal. This config is only valid in Spark3.x");
 
+  public static final ConfigOption<Integer> BLOCKID_SEQUENCE_NO_BITS =
+      ConfigOptions.key("rss.client.blockId.sequenceNoBits")
+          .intType()
+          .defaultValue(BlockIdLayout.DEFAULT.sequenceNoBits)
+          .withDescription(
+              "Block ids contain three fields: the sequence number, the partition id and "
+                  + "the task attempt id. This configures the bits reserved for the sequence "
+                  + "number. Each field can at most have 31 bits, while all fields together "
+                  + "must sum up to 63 bits.");
+
+  public static final ConfigOption<Integer> BLOCKID_PARTITION_ID_BITS =
+      ConfigOptions.key("rss.client.blockId.partitionIdBits")
+          .intType()
+          .defaultValue(BlockIdLayout.DEFAULT.partitionIdBits)
+          .withDescription(
+              "Block ids contain three fields: the sequence number, the partition id and "
+                  + "the task attempt id. This configures the bits reserved for the partition id. "
+                  + "Each field can at most have 31 bits, while all fields together "
+                  + "must sum up to 63 bits.");
+
+  public static final ConfigOption<Integer> BLOCKID_TASK_ATTEMPT_ID_BITS =
+      ConfigOptions.key("rss.client.blockId.taskAttemptIdBits")
+          .intType()
+          .defaultValue(BlockIdLayout.DEFAULT.taskAttemptIdBits)
+          .withDescription(
+              "Block ids contain three fields: the sequence number, the partition id and "
+                  + "the task attempt id. This configures the bits reserved for the task attempt id. "
+                  + "Each field can at most have 31 bits, while all fields together "
+                  + "must sum up to 63 bits.");
+
   public static final ConfigOption<Integer> MAX_CONCURRENCY_PER_PARTITION_TO_WRITE =
       ConfigOptions.key("rss.client.max.concurrency.of.per-partition.write")
           .intType()
@@ -55,6 +96,18 @@ public class RssClientConf {
               "The max concurrency for single partition to write, the value is the max file number "
                   + "for one partition, remote shuffle server should respect this.");
 
+  public static final ConfigOption<Long> RPC_TIMEOUT_MS =
+      ConfigOptions.key("rss.client.rpc.timeout.ms")
+          .longType()
+          .defaultValue(60 * 1000L)
+          .withDescription("Timeout in milliseconds for RPC calls.");
+
+  public static final ConfigOption<Integer> RPC_MAX_ATTEMPTS =
+      ConfigOptions.key("rss.client.rpc.maxAttempts")
+          .intType()
+          .defaultValue(3)
+          .withDescription("When we fail to send RPC calls, we will retry for maxAttempts times.");
+
   public static final ConfigOption<Integer> NETTY_IO_CONNECT_TIMEOUT_MS =
       ConfigOptions.key("rss.client.netty.io.connect.timeout.ms")
           .intType()
@@ -135,4 +188,25 @@ public class RssClientConf {
           .stringType()
           .defaultValue("14m")
           .withDescription("The max data size read from storage");
+
+  public static final ConfigOption<ClientType> RSS_CLIENT_TYPE =
+      ConfigOptions.key("rss.client.type")
+          .enumType(ClientType.class)
+          .defaultValue(ClientType.GRPC)
+          .withDescription("Supports GRPC, GRPC_NETTY");
+
+  public static final ConfigOption<Boolean> RSS_CLIENT_REMOTE_STORAGE_USE_LOCAL_CONF_ENABLED =
+      ConfigOptions.key("rss.client.remote.storage.useLocalConfAsDefault")
+          .booleanType()
+          .defaultValue(false)
+          .withDescription(
+              "This option is only valid when the remote storage path is specified. If ture, "
+                  + "the remote storage conf will use the client side hadoop configuration loaded from the classpath.");
+
+  public static final ConfigOption<Boolean> RSS_CLIENT_BLOCK_SEND_FAILURE_RETRY_ENABLED =
+      ConfigOptions.key("rss.client.blockSendFailureRetry.enabled")
+          .booleanType()
+          .defaultValue(false)
+          .withDescription(
+              "Whether to support rss client block send failure retry, default value is false.");
 }
