@@ -19,7 +19,6 @@ package org.apache.uniffle.common.metrics.prometheus;
 
 import java.util.HashMap;
 import java.util.Map;
-import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 
@@ -31,11 +30,13 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.common.config.RssConf;
+import org.apache.uniffle.common.exception.RssException;
 import org.apache.uniffle.common.metrics.AbstractMetricReporter;
 import org.apache.uniffle.common.util.ThreadUtils;
 
 public class PrometheusPushGatewayMetricReporter extends AbstractMetricReporter {
-  private static final Logger LOG = LoggerFactory.getLogger(PrometheusPushGatewayMetricReporter.class);
+  private static final Logger LOG =
+      LoggerFactory.getLogger(PrometheusPushGatewayMetricReporter.class);
   static final String PUSHGATEWAY_ADDR = "rss.metrics.prometheus.pushgateway.addr";
   static final String GROUPING_KEY = "rss.metrics.prometheus.pushgateway.groupingkey";
   static final String JOB_NAME = "rss.metrics.prometheus.pushgateway.jobname";
@@ -43,7 +44,7 @@ public class PrometheusPushGatewayMetricReporter extends AbstractMetricReporter
   private ScheduledExecutorService scheduledExecutorService;
   private PushGateway pushGateway;
 
-  public PrometheusPushGatewayMetricReporter(RssConf conf, String instanceId)  {
+  public PrometheusPushGatewayMetricReporter(RssConf conf, String instanceId) {
     super(conf, instanceId);
   }
 
@@ -52,28 +53,32 @@ public class PrometheusPushGatewayMetricReporter extends AbstractMetricReporter
     if (pushGateway == null) {
       String address = conf.getString(PUSHGATEWAY_ADDR, null);
       if (StringUtils.isEmpty(address)) {
-        throw new RuntimeException(PUSHGATEWAY_ADDR + " should not be empty!");
+        throw new RssException(PUSHGATEWAY_ADDR + " should not be empty!");
       }
       pushGateway = new PushGateway(address);
     }
     String jobName = conf.getString(JOB_NAME, null);
     if (StringUtils.isEmpty(jobName)) {
-      throw new RuntimeException(JOB_NAME + " should not be empty!");
+      throw new RssException(JOB_NAME + " should not be empty!");
     }
     Map<String, String> groupingKey = parseGroupingKey(conf.getString(GROUPING_KEY, ""));
     groupingKey.put("instance", instanceId);
     int reportInterval = conf.getInteger(REPORT_INTEVAL, 10);
-    scheduledExecutorService = Executors.newScheduledThreadPool(1,
-        ThreadUtils.getThreadFactory("PrometheusPushGatewayMetricReporter-%d"));
-    scheduledExecutorService.scheduleWithFixedDelay(() -> {
-      for (CollectorRegistry registry : registryList) {
-        try {
-          pushGateway.pushAdd(registry, jobName, groupingKey);
-        } catch (Throwable e) {
-          LOG.error("Failed to send metrics to push gateway.", e);
-        }
-      }
-    }, 0, reportInterval, TimeUnit.SECONDS);
+    scheduledExecutorService =
+        ThreadUtils.getDaemonSingleThreadScheduledExecutor("PrometheusPushGatewayMetricReporter");
+    scheduledExecutorService.scheduleWithFixedDelay(
+        () -> {
+          for (CollectorRegistry registry : registryList) {
+            try {
+              pushGateway.pushAdd(registry, jobName, groupingKey);
+            } catch (Throwable e) {
+              LOG.error("Failed to send metrics to push gateway.", e);
+            }
+          }
+        },
+        0,
+        reportInterval,
+        TimeUnit.SECONDS);
   }
 
   @Override
@@ -101,8 +106,7 @@ public class PrometheusPushGatewayMetricReporter extends AbstractMetricReporter
 
         String labelKey = kv.substring(0, idx);
         String labelValue = kv.substring(idx + 1);
-        if (StringUtils.isEmpty(labelKey)
-            || StringUtils.isEmpty(labelValue)) {
+        if (StringUtils.isEmpty(labelKey) || StringUtils.isEmpty(labelValue)) {
           LOG.warn(
               "Invalid groupingKey {labelKey:{}, labelValue:{}} must not be empty",
               labelKey,
