@@ -27,13 +27,14 @@ public class CoordinatorConfTest {
 
   @Test
   public void test() {
-    final String filePath = Objects.requireNonNull(
-        getClass().getClassLoader().getResource("coordinator.conf")).getFile();
+    final String filePath =
+        Objects.requireNonNull(getClass().getClassLoader().getResource("coordinator.conf"))
+            .getFile();
     CoordinatorConf conf = new CoordinatorConf(filePath);
 
     // test base conf
     assertEquals(9527, conf.getInteger(CoordinatorConf.RPC_SERVER_PORT));
-    assertEquals("testRpc", conf.getString(CoordinatorConf.RPC_SERVER_TYPE));
+    assertEquals("GRPC", conf.get(CoordinatorConf.RPC_SERVER_TYPE).name());
     assertEquals(9526, conf.getInteger(CoordinatorConf.JETTY_HTTP_PORT));
 
     // test coordinator specific conf
@@ -42,12 +43,12 @@ public class CoordinatorConfTest {
     assertEquals(123, conf.getLong(CoordinatorConf.COORDINATOR_HEARTBEAT_TIMEOUT));
 
     // test default conf
-    assertEquals("PARTITION_BALANCE", conf.get(CoordinatorConf.COORDINATOR_ASSIGNMENT_STRATEGY).name());
+    assertEquals(
+        "PARTITION_BALANCE", conf.get(CoordinatorConf.COORDINATOR_ASSIGNMENT_STRATEGY).name());
     assertEquals(256, conf.getInteger(CoordinatorConf.JETTY_CORE_POOL_SIZE));
     assertEquals(60 * 1000, conf.getLong(CoordinatorConf.COORDINATOR_EXCLUDE_NODES_CHECK_INTERVAL));
 
     // test custom keys defined in plugins
     assertEquals("v1", conf.getString("plugin.custom.key", null));
   }
-
 }
