@@ -29,7 +29,8 @@ public class AQESkewedJoinWithLocalOrderTest extends AQESkewedJoinTest {
   @Override
   public void updateSparkConfCustomer(SparkConf sparkConf) {
     sparkConf.set(RssSparkConfig.RSS_STORAGE_TYPE.key(), StorageType.LOCALFILE.name());
-    sparkConf.set("spark." + RssClientConf.DATA_DISTRIBUTION_TYPE.key(),
+    sparkConf.set(
+        "spark." + RssClientConf.DATA_DISTRIBUTION_TYPE.key(),
         ShuffleDataDistributionType.LOCAL_ORDER.name());
   }
 }
