@@ -17,10 +17,10 @@
 
 package org.apache.uniffle.client.request;
 
-
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.uniffle.common.ServerStatus;
 import org.apache.uniffle.common.storage.StorageInfo;
 
 public class RssSendHeartBeatRequest {
@@ -34,8 +34,9 @@ public class RssSendHeartBeatRequest {
   private final int eventNumInFlush;
   private final Set<String> tags;
   private final long timeout;
-  private final boolean isHealthy;
+  private final ServerStatus serverStatus;
   private final Map<String, StorageInfo> storageInfo;
+  private final int nettyPort;
 
   public RssSendHeartBeatRequest(
       String shuffleServerId,
@@ -47,8 +48,9 @@ public class RssSendHeartBeatRequest {
       int eventNumInFlush,
       long timeout,
       Set<String> tags,
-      boolean isHealthy,
-      Map<String, StorageInfo> storageInfo) {
+      ServerStatus serverStatus,
+      Map<String, StorageInfo> storageInfo,
+      int nettyPort) {
     this.shuffleServerId = shuffleServerId;
     this.shuffleServerIp = shuffleServerIp;
     this.shuffleServerPort = shuffleServerPort;
@@ -58,8 +60,9 @@ public class RssSendHeartBeatRequest {
     this.eventNumInFlush = eventNumInFlush;
     this.tags = tags;
     this.timeout = timeout;
-    this.isHealthy = isHealthy;
+    this.serverStatus = serverStatus;
     this.storageInfo = storageInfo;
+    this.nettyPort = nettyPort;
   }
 
   public String getShuffleServerId() {
@@ -98,11 +101,15 @@ public class RssSendHeartBeatRequest {
     return tags;
   }
 
-  public boolean isHealthy() {
-    return isHealthy;
+  public ServerStatus getServerStatus() {
+    return serverStatus;
   }
 
   public Map<String, StorageInfo> getStorageInfo() {
     return storageInfo;
   }
+
+  public int getNettyPort() {
+    return nettyPort;
+  }
 }
