@@ -31,6 +31,7 @@ import com.google.protobuf.UnsafeByteOperations;
 import io.grpc.Context;
 import io.grpc.Status;
 import io.grpc.stub.StreamObserver;
+import io.netty.buffer.ByteBuf;
 import org.apache.commons.lang3.StringUtils;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 import org.slf4j.Logger;
@@ -46,7 +47,12 @@ import org.apache.uniffle.common.ShufflePartitionedBlock;
 import org.apache.uniffle.common.ShufflePartitionedData;
 import org.apache.uniffle.common.config.RssBaseConf;
 import org.apache.uniffle.common.exception.FileNotFoundException;
+import org.apache.uniffle.common.exception.NoBufferException;
+import org.apache.uniffle.common.exception.NoBufferForHugePartitionException;
+import org.apache.uniffle.common.exception.NoRegisterException;
 import org.apache.uniffle.common.rpc.StatusCode;
+import org.apache.uniffle.common.util.BlockIdLayout;
+import org.apache.uniffle.common.util.ByteBufUtils;
 import org.apache.uniffle.common.util.RssUtils;
 import org.apache.uniffle.proto.RssProtos;
 import org.apache.uniffle.proto.RssProtos.AppHeartBeatRequest;
@@ -81,7 +87,6 @@ import org.apache.uniffle.proto.RssProtos.ShuffleRegisterRequest;
 import org.apache.uniffle.proto.RssProtos.ShuffleRegisterResponse;
 import org.apache.uniffle.proto.ShuffleServerGrpc.ShuffleServerImplBase;
 import org.apache.uniffle.server.buffer.PreAllocatedBufferInfo;
-import org.apache.uniffle.server.buffer.RequireBufferStatusCode;
 import org.apache.uniffle.storage.common.Storage;
 import org.apache.uniffle.storage.common.StorageReadMetrics;
 import org.apache.uniffle.storage.util.ShuffleStorageUtils;
@@ -95,6 +100,30 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     this.shuffleServer = shuffleServer;
   }
 
+  @Override
+  public void unregisterShuffleByAppId(
+      RssProtos.ShuffleUnregisterByAppIdRequest request,
+      StreamObserver<RssProtos.ShuffleUnregisterByAppIdResponse> responseStreamObserver) {
+    String appId = request.getAppId();
+
+    StatusCode result = StatusCode.SUCCESS;
+    String responseMessage = "OK";
+    try {
+      shuffleServer.getShuffleTaskManager().removeShuffleDataAsync(appId);
+
+    } catch (Exception e) {
+      result = StatusCode.INTERNAL_ERROR;
+    }
+
+    RssProtos.ShuffleUnregisterByAppIdResponse reply =
+        RssProtos.ShuffleUnregisterByAppIdResponse.newBuilder()
+            .setStatus(result.toProto())
+            .setRetMsg(responseMessage)
+            .build();
+    responseStreamObserver.onNext(reply);
+    responseStreamObserver.onCompleted();
+  }
+
   @Override
   public void unregisterShuffle(
       RssProtos.ShuffleUnregisterRequest request,
@@ -228,6 +257,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       final long start = System.currentTimeMillis();
       List<ShufflePartitionedData> shufflePartitionedData = toPartitionedData(req);
       long alreadyReleasedSize = 0;
+      boolean hasFailureOccurred = false;
       for (ShufflePartitionedData spd : shufflePartitionedData) {
         String shuffleDataInfo =
             "appId["
@@ -247,6 +277,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
                     + ret;
             LOG.error(errorMsg);
             responseMessage = errorMsg;
+            hasFailureOccurred = true;
             break;
           } else {
             long toReleasedSize = spd.getTotalBlockSize();
@@ -265,7 +296,14 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
           ret = StatusCode.INTERNAL_ERROR;
           responseMessage = errorMsg;
           LOG.error(errorMsg);
+          hasFailureOccurred = true;
           break;
+        } finally {
+          if (hasFailureOccurred) {
+            shuffleServer
+                .getShuffleBufferManager()
+                .releaseMemory(spd.getTotalBlockSize(), false, false);
+          }
         }
       }
       // since the required buffer id is only used once, the shuffle client would try to require
@@ -285,18 +323,20 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       shuffleServer
           .getGrpcMetrics()
           .recordProcessTime(ShuffleServerGrpcMetrics.SEND_SHUFFLE_DATA_METHOD, costTime);
-      LOG.debug(
-          "Cache Shuffle Data for appId["
-              + appId
-              + "], shuffleId["
-              + shuffleId
-              + "], cost "
-              + costTime
-              + " ms with "
-              + shufflePartitionedData.size()
-              + " blocks and "
-              + requireSize
-              + " bytes");
+      if (LOG.isDebugEnabled()) {
+        LOG.debug(
+            "Cache Shuffle Data for appId["
+                + appId
+                + "], shuffleId["
+                + shuffleId
+                + "], cost "
+                + costTime
+                + " ms with "
+                + shufflePartitionedData.size()
+                + " blocks and "
+                + requireSize
+                + " bytes");
+      }
     } else {
       reply =
           SendShuffleDataResponse.newBuilder()
@@ -326,14 +366,16 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         throw new IllegalStateException("AppId " + appId + " was removed already");
       }
       commitCount = shuffleServer.getShuffleTaskManager().updateAndGetCommitCount(appId, shuffleId);
-      LOG.debug(
-          "Get commitShuffleTask request for appId["
-              + appId
-              + "], shuffleId["
-              + shuffleId
-              + "], currentCommitted["
-              + commitCount
-              + "]");
+      if (LOG.isDebugEnabled()) {
+        LOG.debug(
+            "Get commitShuffleTask request for appId["
+                + appId
+                + "], shuffleId["
+                + shuffleId
+                + "], currentCommitted["
+                + commitCount
+                + "]");
+      }
     } catch (Exception e) {
       status = StatusCode.INTERNAL_ERROR;
       msg = "Error happened when commit for appId[" + appId + "], shuffleId[" + shuffleId + "]";
@@ -386,27 +428,32 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
   public void requireBuffer(
       RequireBufferRequest request, StreamObserver<RequireBufferResponse> responseObserver) {
     String appId = request.getAppId();
-    long requireBufferId;
-    if (StringUtils.isEmpty(appId)) {
-      // To be compatible with older client version
-      requireBufferId =
-          shuffleServer.getShuffleTaskManager().requireBuffer(request.getRequireSize());
-    } else {
-      requireBufferId =
-          shuffleServer
-              .getShuffleTaskManager()
-              .requireBuffer(
-                  appId,
-                  request.getShuffleId(),
-                  request.getPartitionIdsList(),
-                  request.getRequireSize());
-    }
-
+    long requireBufferId = -1;
     StatusCode status = StatusCode.SUCCESS;
-    if (requireBufferId == RequireBufferStatusCode.NO_BUFFER.statusCode()) {
+    try {
+      if (StringUtils.isEmpty(appId)) {
+        // To be compatible with older client version
+        requireBufferId =
+            shuffleServer.getShuffleTaskManager().requireBuffer(request.getRequireSize());
+      } else {
+        requireBufferId =
+            shuffleServer
+                .getShuffleTaskManager()
+                .requireBuffer(
+                    appId,
+                    request.getShuffleId(),
+                    request.getPartitionIdsList(),
+                    request.getRequireSize());
+      }
+    } catch (NoBufferException e) {
       status = StatusCode.NO_BUFFER;
+      ShuffleServerMetrics.counterTotalRequireBufferFailedForRegularPartition.inc();
       ShuffleServerMetrics.counterTotalRequireBufferFailed.inc();
-    } else if (requireBufferId == RequireBufferStatusCode.NO_REGISTER.statusCode()) {
+    } catch (NoBufferForHugePartitionException e) {
+      status = StatusCode.NO_BUFFER_FOR_HUGE_PARTITION;
+      ShuffleServerMetrics.counterTotalRequireBufferFailedForHugePartition.inc();
+      ShuffleServerMetrics.counterTotalRequireBufferFailed.inc();
+    } catch (NoRegisterException e) {
       status = StatusCode.NO_REGISTER;
       ShuffleServerMetrics.counterTotalRequireBufferFailed.inc();
     }
@@ -485,6 +532,11 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     String appId = request.getAppId();
     int shuffleId = request.getShuffleId();
     int partitionId = request.getPartitionId();
+    BlockIdLayout blockIdLayout =
+        BlockIdLayout.from(
+            request.getBlockIdLayout().getSequenceNoBits(),
+            request.getBlockIdLayout().getPartitionIdBits(),
+            request.getBlockIdLayout().getTaskAttemptIdBits());
     StatusCode status = StatusCode.SUCCESS;
     String msg = "OK";
     GetShuffleResultResponse reply;
@@ -497,7 +549,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       serializedBlockIds =
           shuffleServer
               .getShuffleTaskManager()
-              .getFinishedBlockIds(appId, shuffleId, Sets.newHashSet(partitionId));
+              .getFinishedBlockIds(appId, shuffleId, Sets.newHashSet(partitionId), blockIdLayout);
       if (serializedBlockIds == null) {
         status = StatusCode.INTERNAL_ERROR;
         msg = "Can't get shuffle result for " + requestInfo;
@@ -528,6 +580,11 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     String appId = request.getAppId();
     int shuffleId = request.getShuffleId();
     List<Integer> partitionsList = request.getPartitionsList();
+    BlockIdLayout blockIdLayout =
+        BlockIdLayout.from(
+            request.getBlockIdLayout().getSequenceNoBits(),
+            request.getBlockIdLayout().getPartitionIdBits(),
+            request.getBlockIdLayout().getTaskAttemptIdBits());
 
     StatusCode status = StatusCode.SUCCESS;
     String msg = "OK";
@@ -541,7 +598,8 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       serializedBlockIds =
           shuffleServer
               .getShuffleTaskManager()
-              .getFinishedBlockIds(appId, shuffleId, Sets.newHashSet(partitionsList));
+              .getFinishedBlockIds(
+                  appId, shuffleId, Sets.newHashSet(partitionsList), blockIdLayout);
       if (serializedBlockIds == null) {
         status = StatusCode.INTERNAL_ERROR;
         msg = "Can't get shuffle result for " + requestInfo;
@@ -616,7 +674,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       storage.updateReadMetrics(new StorageReadMetrics(appId, shuffleId));
     }
 
-    if (shuffleServer.getShuffleBufferManager().requireReadMemoryWithRetry(length)) {
+    if (shuffleServer.getShuffleBufferManager().requireReadMemory(length)) {
       try {
         long start = System.currentTimeMillis();
         sdr =
@@ -664,7 +722,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         shuffleServer.getShuffleBufferManager().releaseReadMemory(length);
       }
     } else {
-      status = StatusCode.INTERNAL_ERROR;
+      status = StatusCode.NO_BUFFER;
       msg = "Can't require memory to get shuffle data";
       LOG.error(msg + " for " + requestInfo);
       reply =
@@ -708,7 +766,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         shuffleServer
             .getShuffleServerConf()
             .getLong(ShuffleServerConf.SERVER_SHUFFLE_INDEX_SIZE_HINT);
-    if (shuffleServer.getShuffleBufferManager().requireReadMemoryWithRetry(assumedFileSize)) {
+    if (shuffleServer.getShuffleBufferManager().requireReadMemory(assumedFileSize)) {
       ShuffleIndexResult shuffleIndexResult = null;
       try {
         long start = System.currentTimeMillis();
@@ -754,7 +812,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         shuffleServer.getShuffleBufferManager().releaseReadMemory(assumedFileSize);
       }
     } else {
-      status = StatusCode.INTERNAL_ERROR;
+      status = StatusCode.NO_BUFFER;
       msg = "Can't require memory to get shuffle index";
       LOG.error(msg + " for " + requestInfo);
       reply =
@@ -795,7 +853,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         "appId[" + appId + "], shuffleId[" + shuffleId + "], partitionId[" + partitionId + "]";
 
     // todo: if can get the exact memory size?
-    if (shuffleServer.getShuffleBufferManager().requireReadMemoryWithRetry(readBufferSize)) {
+    if (shuffleServer.getShuffleBufferManager().requireReadMemory(readBufferSize)) {
       ShuffleDataResult shuffleDataResult = null;
       try {
         Roaring64NavigableMap expectedTaskIds = null;
@@ -857,7 +915,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         shuffleServer.getShuffleBufferManager().releaseReadMemory(readBufferSize);
       }
     } else {
-      status = StatusCode.INTERNAL_ERROR;
+      status = StatusCode.NO_BUFFER;
       msg = "Can't require memory to get in memory shuffle data";
       LOG.error(msg + " for " + requestInfo);
       reply =
@@ -892,6 +950,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     ShufflePartitionedBlock[] ret = new ShufflePartitionedBlock[blocks.size()];
     int i = 0;
     for (ShuffleBlock block : blocks) {
+      ByteBuf data = ByteBufUtils.byteStringToByteBuf(block.getData());
       ret[i] =
           new ShufflePartitionedBlock(
               block.getLength(),
@@ -899,7 +958,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
               block.getCrc(),
               block.getBlockId(),
               block.getTaskAttemptId(),
-              block.getData().toByteArray());
+              data);
       i++;
     }
     return ret;
