@@ -53,10 +53,16 @@ public abstract class BufferTestBase {
   protected ShufflePartitionedData createData(int partitionId, int taskAttemptId, int len) {
     byte[] buf = new byte[len];
     new Random().nextBytes(buf);
-    ShufflePartitionedBlock block = new ShufflePartitionedBlock(
-        len, len, ChecksumUtils.getCrc32(buf), atomBlockId.incrementAndGet(), taskAttemptId, buf);
-    ShufflePartitionedData data = new ShufflePartitionedData(
-        partitionId, new ShufflePartitionedBlock[]{block});
+    ShufflePartitionedBlock block =
+        new ShufflePartitionedBlock(
+            len,
+            len,
+            ChecksumUtils.getCrc32(buf),
+            atomBlockId.incrementAndGet(),
+            taskAttemptId,
+            buf);
+    ShufflePartitionedData data =
+        new ShufflePartitionedData(partitionId, new ShufflePartitionedBlock[] {block});
     return data;
   }
 }
