@@ -25,7 +25,7 @@ import java.util.Collections;
 import java.util.List;
 
 import com.google.common.collect.Lists;
-import org.apache.commons.lang.exception.ExceptionUtils;
+import org.apache.commons.lang3.exception.ExceptionUtils;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.util.hash.MurmurHash;
@@ -33,19 +33,19 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.common.BufferSegment;
+import org.apache.uniffle.common.exception.RssException;
 import org.apache.uniffle.common.util.Constants;
 import org.apache.uniffle.storage.common.FileBasedShuffleSegment;
 import org.apache.uniffle.storage.handler.impl.DataFileSegment;
-import org.apache.uniffle.storage.handler.impl.HdfsFileWriter;
+import org.apache.uniffle.storage.handler.impl.HadoopFileWriter;
 
 public class ShuffleStorageUtils {
 
-  static final String HDFS_PATH_SEPARATOR = "/";
-  static final String HDFS_DIRNAME_SEPARATOR = "-";
+  static final String HADOOP_PATH_SEPARATOR = "/";
+  static final String HADOOP_DIRNAME_SEPARATOR = "-";
   private static final Logger LOG = LoggerFactory.getLogger(ShuffleStorageUtils.class);
 
-  private ShuffleStorageUtils() {
-  }
+  private ShuffleStorageUtils() {}
 
   public static String generateDataFileName(String fileNamePrefix) {
     return fileNamePrefix + Constants.SHUFFLE_DATA_FILE_SUFFIX;
@@ -55,26 +55,23 @@ public class ShuffleStorageUtils {
     return fileNamePrefix + Constants.SHUFFLE_INDEX_FILE_SUFFIX;
   }
 
-  public static String generateAbsoluteFilePrefix(String base, String key, int partition, String id) {
-    return String.join(
-        HDFS_PATH_SEPARATOR,
-        base,
-        key,
-        String.join(HDFS_DIRNAME_SEPARATOR, String.valueOf(partition), String.valueOf(partition)),
-        id);
-  }
-
   public static List<DataFileSegment> mergeSegments(
       String path, List<FileBasedShuffleSegment> segments, int readBufferSize) {
     List<DataFileSegment> dataFileSegments = Lists.newArrayList();
     if (segments != null && !segments.isEmpty()) {
       if (segments.size() == 1) {
         List<BufferSegment> bufferSegments = Lists.newArrayList();
-        bufferSegments.add(new BufferSegment(segments.get(0).getBlockId(), 0,
-            segments.get(0).getLength(), segments.get(0).getUncompressLength(), segments.get(0).getCrc(),
-            segments.get(0).getTaskAttemptId()));
-        dataFileSegments.add(new DataFileSegment(
-            path, segments.get(0).getOffset(), segments.get(0).getLength(), bufferSegments));
+        bufferSegments.add(
+            new BufferSegment(
+                segments.get(0).getBlockId(),
+                0,
+                segments.get(0).getLength(),
+                segments.get(0).getUncompressLength(),
+                segments.get(0).getCrc(),
+                segments.get(0).getTaskAttemptId()));
+        dataFileSegments.add(
+            new DataFileSegment(
+                path, segments.get(0).getOffset(), segments.get(0).getLength(), bufferSegments));
       } else {
         Collections.sort(segments);
         long start = -1;
@@ -83,10 +80,11 @@ public class ShuffleStorageUtils {
         long lastPosition = Long.MAX_VALUE;
         List<BufferSegment> bufferSegments = Lists.newArrayList();
         for (FileBasedShuffleSegment segment : segments) {
-          // check if there has expected skip range, eg, [20, 100], [1000, 1001] and the skip range is [101, 999]
+          // check if there has expected skip range, eg, [20, 100], [1000, 1001] and the skip range
+          // is [101, 999]
           if (start > -1 && segment.getOffset() - lastPosition > skipThreshold) {
-            dataFileSegments.add(new DataFileSegment(
-                path, start, (int) (lastPosition - start), bufferSegments));
+            dataFileSegments.add(
+                new DataFileSegment(path, start, (int) (lastPosition - start), bufferSegments));
             start = -1;
           }
           // previous FileBasedShuffleSegment are merged, start new merge process
@@ -95,18 +93,24 @@ public class ShuffleStorageUtils {
             start = segment.getOffset();
           }
           latestPosition = segment.getOffset() + segment.getLength();
-          bufferSegments.add(new BufferSegment(segment.getBlockId(),
-              segment.getOffset() - start, segment.getLength(),
-              segment.getUncompressLength(), segment.getCrc(), segment.getTaskAttemptId()));
+          bufferSegments.add(
+              new BufferSegment(
+                  segment.getBlockId(),
+                  segment.getOffset() - start,
+                  segment.getLength(),
+                  segment.getUncompressLength(),
+                  segment.getCrc(),
+                  segment.getTaskAttemptId()));
           if (latestPosition - start >= readBufferSize) {
-            dataFileSegments.add(new DataFileSegment(
-                path, start, (int) (latestPosition - start), bufferSegments));
+            dataFileSegments.add(
+                new DataFileSegment(path, start, (int) (latestPosition - start), bufferSegments));
             start = -1;
           }
           lastPosition = latestPosition;
         }
         if (start > -1) {
-          dataFileSegments.add(new DataFileSegment(path, start, (int) (lastPosition - start), bufferSegments));
+          dataFileSegments.add(
+              new DataFileSegment(path, start, (int) (lastPosition - start), bufferSegments));
         }
       }
     }
@@ -114,45 +118,31 @@ public class ShuffleStorageUtils {
   }
 
   public static String getShuffleDataPath(String appId, int shuffleId) {
-    return String.join(
-        HDFS_PATH_SEPARATOR,
-        appId,
-        String.valueOf(shuffleId));
+    return String.join(HADOOP_PATH_SEPARATOR, appId, String.valueOf(shuffleId));
   }
 
   public static String getShuffleDataPath(String appId, int shuffleId, int start, int end) {
     return String.join(
-        HDFS_PATH_SEPARATOR,
+        HADOOP_PATH_SEPARATOR,
         appId,
         String.valueOf(shuffleId),
-        String.join(HDFS_DIRNAME_SEPARATOR, String.valueOf(start), String.valueOf(end)));
-  }
-
-  public static String getUploadShuffleDataPath(String appId, int shuffleId, int partitionId) {
-    return String.join(
-        HDFS_PATH_SEPARATOR,
-        appId,
-        String.valueOf(shuffleId),
-        String.valueOf(partitionId));
+        String.join(HADOOP_DIRNAME_SEPARATOR, String.valueOf(start), String.valueOf(end)));
   }
 
   public static String getCombineDataPath(String appId, int shuffleId) {
-    return String.join(
-        HDFS_PATH_SEPARATOR,
-        appId,
-        String.valueOf(shuffleId),
-        "combine");
+    return String.join(HADOOP_PATH_SEPARATOR, appId, String.valueOf(shuffleId), "combine");
   }
 
   public static String getFullShuffleDataFolder(String basePath, String subPath) {
-    return String.join(HDFS_PATH_SEPARATOR, basePath, subPath);
+    return String.join(HADOOP_PATH_SEPARATOR, basePath, subPath);
   }
 
   public static String getShuffleDataPathWithRange(
-      String appId, int shuffleId, int partitionId,
-      int partitionNumPerRange, int partitionNum) {
-    int prNum = partitionNum % partitionNumPerRange == 0
-        ? partitionNum / partitionNumPerRange : partitionNum / partitionNumPerRange + 1;
+      String appId, int shuffleId, int partitionId, int partitionNumPerRange, int partitionNum) {
+    int prNum =
+        partitionNum % partitionNumPerRange == 0
+            ? partitionNum / partitionNumPerRange
+            : partitionNum / partitionNumPerRange + 1;
     for (int i = 0; i < prNum; i++) {
       int start = i * partitionNumPerRange;
       int end = (i + 1) * partitionNumPerRange - 1;
@@ -160,19 +150,32 @@ public class ShuffleStorageUtils {
         return getShuffleDataPath(appId, shuffleId, start, end);
       }
     }
-    throw new RuntimeException("Can't generate ShuffleData Path for appId[" + appId + "], shuffleId["
-        + shuffleId + "], partitionId[" + partitionId + "], partitionNumPerRange[" + partitionNumPerRange
-        + "], partitionNum[" + partitionNum + "]");
+    throw new RssException(
+        "Can't generate ShuffleData Path for appId["
+            + appId
+            + "], shuffleId["
+            + shuffleId
+            + "], partitionId["
+            + partitionId
+            + "], partitionNumPerRange["
+            + partitionNumPerRange
+            + "], partitionNum["
+            + partitionNum
+            + "]");
   }
 
-  public static int[] getPartitionRange(int partitionId, int partitionNumPerRange, int partitionNum) {
+  public static int[] getPartitionRange(
+      int partitionId, int partitionNumPerRange, int partitionNum) {
     int[] range = null;
     int prNum = partitionId / partitionNumPerRange;
     if (partitionId < 0 || partitionId >= partitionNum) {
-      LOG.warn("Invalid partitionId. partitionId:{} ,partitionNumPerRange: {}, partitionNum: {}",
-              partitionId, partitionNumPerRange, partitionNum);
+      LOG.warn(
+          "Invalid partitionId. partitionId:{} ,partitionNumPerRange: {}, partitionNum: {}",
+          partitionId,
+          partitionNumPerRange,
+          partitionNum);
     } else {
-      range = new int[]{partitionNumPerRange * prNum, partitionNumPerRange * (prNum + 1) - 1};
+      range = new int[] {partitionNumPerRange * prNum, partitionNumPerRange * (prNum + 1) - 1};
     }
     return range;
   }
@@ -186,7 +189,8 @@ public class ShuffleStorageUtils {
     return index;
   }
 
-  public static void createDirIfNotExist(FileSystem fileSystem, String pathString) throws IOException {
+  public static void createDirIfNotExist(FileSystem fileSystem, String pathString)
+      throws IOException {
     Path path = new Path(pathString);
     try {
       if (!fileSystem.exists(path)) {
@@ -195,22 +199,15 @@ public class ShuffleStorageUtils {
     } catch (IOException ioe) {
       // if folder exist, ignore the exception
       if (!fileSystem.exists(path)) {
-        LOG.error("Can't create shuffle folder {}, {}", pathString, ExceptionUtils.getStackTrace(ioe));
+        LOG.error(
+            "Can't create shuffle folder {}, {}", pathString, ExceptionUtils.getStackTrace(ioe));
         throw ioe;
       }
     }
   }
 
-  // index file header is $PartitionNum | [($PartitionId | $PartitionFileLength | $PartitionDataFileLength), ] | $CRC
-  public static long getIndexFileHeaderLen(int partitionNum) {
-    return 4 + (4 + 8 + 8) * (long) partitionNum + 8;
-  }
-
-  public static int getHeaderCrcLen() {
-    return 8;
-  }
-
-  public static long uploadFile(File file, HdfsFileWriter writer, int bufferSize) throws IOException {
+  public static long uploadFile(File file, HadoopFileWriter writer, int bufferSize)
+      throws IOException {
     try (FileInputStream inputStream = new FileInputStream(file)) {
       return writer.copy(inputStream, bufferSize);
     } catch (IOException e) {
